#!/usr/bin/env python3
"""
DNS Debugging Tool
Parameters:
    - IP: DNS server IP
    - port: DNS server port (default: 53)
Additional parameters in comments:
    - query: Domain to query
    - record_type: DNS record type
"""
import dns.resolver
import dns.query
import dns.message
import dns.rdatatype
import dns.name
import argparse
import time
from datetime import datetime

class DNSDebugger:
    def __init__(self, server_ip, port=53):
        self.server_ip = server_ip
        self.port = port
        self.resolver = dns.resolver.Resolver()
        self.resolver.nameservers = [server_ip]
        self.resolver.port = port
        
    def test_record_types(self, domain):
        """Test different DNS record types"""
        record_types = ['A', 'AAAA', 'MX', 'NS', 'TXT', 'SOA', 'CNAME', 'PTR']
        results = {}
        
        print(f"\nTesting DNS records for {domain}")
        print(f"Server: {self.server_ip}:{self.port}")
        print("-" * 50)
        
        for record_type in record_types:
            try:
                start_time = time.time()
                answers = self.resolver.resolve(domain, record_type)
                response_time = (time.time() - start_time) * 1000
                
                results[record_type] = {
                    'status': 'Found',
                    'count': len(answers),
                    'ttl': answers.rrset.ttl,
                    'response_time': response_time,
                    'data': [str(rdata) for rdata in answers]
                }
                
                print(f"\n{record_type} Record:")
                print(f"Response Time: {response_time:.2f}ms")
                print(f"TTL: {answers.rrset.ttl}")
                for rdata in answers:
                    print(f"  → {rdata}")
                    
            except dns.resolver.NoAnswer:
                results[record_type] = {'status': 'No records found'}
                print(f"\n{record_type} Record: No records found")
            except dns.resolver.NXDOMAIN:
                results[record_type] = {'status': 'Domain does not exist'}
                print(f"\n{record_type} Record: Domain does not exist")
            except Exception as e:
                results[record_type] = {'status': f'Error: {str(e)}'}
                print(f"\n{record_type} Record: Error - {str(e)}")
                
        return results
        
    def test_authoritative_servers(self, domain):
        """Test authoritative nameservers"""
        print("\nTesting Authoritative Nameservers")
        print("-" * 50)
        
        try:
            ns_records = self.resolver.resolve(domain, 'NS')
            for ns in ns_records:
                ns_ip = self.resolver.resolve(str(ns), 'A')[0]
                print(f"\nNameserver: {ns}")
                print(f"IP Address: {ns_ip}")
                
                # Test response from this nameserver
                try:
                    query = dns.message.make_query(domain, dns.rdatatype.A)
                    start_time = time.time()
                    response = dns.query.udp(query, str(ns_ip), timeout=5)
                    response_time = (time.time() - start_time) * 1000
                    
                    print(f"Response Time: {response_time:.2f}ms")
                    print(f"Response Code: {dns.rcode.to_text(response.rcode())}")
                    print(f"Authoritative: {'AA' in response.flags}")
                except Exception as e:
                    print(f"Error querying nameserver: {str(e)}")
                    
        except Exception as e:
            print(f"Error getting nameservers: {str(e)}")
            
    def perform_debug(self, domain):
        print(f"Starting DNS debug session at {datetime.now()}")
        
        # Test basic connectivity
        print("\nTesting DNS server connectivity...")
        try:
            query = dns.message.make_query(domain, dns.rdatatype.A)
            response = dns.query.udp(query, self.server_ip, timeout=5)
            print(f"Server {self.server_ip} is responding")
        except Exception as e:
            print(f"Error connecting to DNS server: {str(e)}")
            return
            
        # Test record types
        self.test_record_types(domain)
        
        # Test authoritative servers
        self.test_authoritative_servers(domain)
        
        print("\nDebug session complete.")

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="DNS Debugging Tool")
    parser.add_argument("ip", help="DNS server IP")
    parser.add_argument("domain", help="Domain to query")
    parser.add_argument("--port", type=int, default=53,
                      help="DNS server port (default: 53)")
    args = parser.parse_args()
    
    debugger = DNSDebugger(args.ip, args.port)
    debugger.perform_debug(args.domain)
